% MAIN FILE
% date: march 2022

% This file imports simulated data from the csv file 'testdata_return.csv' 
% and 'testdata_rcov_vech', which contains simulated open-to-close returns 
% (T = 1000, k = 5), and the vech of a realized covariance matrix for each t, t = 1,...1000

% Then the file estimates parameters of the HEAVY GAS tF model
% proposed in  "New HEAVY Models for Fat-Tailed Realized Covariances and Returns" by Opschoor et al. (2018).

% The parameters are stored in the cell array para_cell
% If required, the file alse computes the standard errors using the
% sandwich estimator of the covariance matrix.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% import data and gather T and k
return_mat      = readmatrix('testdata_return.csv');
RC_mat_vech     = readmatrix('testdata_rcov_vech.csv');
[T,k]           = size(return_mat);

% put to if you would like to compute standard errors based on the sandwich
% covariance matrix
ind_se     = 1;

% transform the RC_vech into a threeway matrix (k times k times T)
RC_mat = zeros(k,k,T);
for t = 1:T
   RC_mat(:,:,t) = Admin.Vech2Sym(k,RC_mat_vech(t,:));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% Matlab routine to set up constrained optimization %%%%%%%%

options = optimset('fmincon');
options = optimset(options, 'Display', 'off');
options = optimset(options, 'Diagnostics', 'off');
options = optimset(options, 'Algorithm', 'sqp');
options.MaxIter     = 30000;
options.MaxFunEvals = 30000;

% Create starting values and lower/upper bounds for the contrained optimization
para_start_GAS_tF  = [0.02 0.8 15 25 25]';

% lower and upper bounds
Lb = [ 0.001 0.0001 2.01 k+1+0.01 k+1+0.01]';
Rb = [ 3.0 0.99999 1000 1000 1000]';

% create a empty cell array for parameters, standard errors (optional) and strings (will be used later on to print the
% results)
para_cell = cell(1,1);
if ind_se==1
    se_para_cell = cell(1,1);
end

para_cell_str{1}= {'A','B', 'nu_0','nu_1','nu_2'};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Estimate the model parameters in three steps:
% 1) Define the function (handle)
% 2) Minimize the MINUS loglikelihood using fmincon
% 3) Store the estimated parameters, loglikelihood and Exit Flag in a cell-array
% 4) (OPTIONAL: compute the standard errors and store these in a
% cell-array)

% NB: Flag= 2 means that we have found a local optimum

% step 1 2 and 3
f_GAS_tF                                 = @(params)LogLik_GAS_tF_CT(k, T, params, return_mat, RC_mat);
[theta_GAS_tF, fval_GAS_tF, Flag_GAS_tF] = fmincon(f_GAS_tF, para_start_GAS_tF,[],[],[],[],Lb,Rb,[],options);
para_cell{1}                             = [theta_GAS_tF', -fval_GAS_tF, Flag_GAS_tF];

% used to print the results
f_handle  = f_GAS_tF;
theta_opt = theta_GAS_tF;
max_LogLik = -fval_GAS_tF;

nr_para = length(theta_opt);

% compute the standard errors (using the sandwich cov matrix)
if ind_se==1
    [~,se_S] = Admin.Compute_se_errors_ML(f_handle,theta_opt,T);
    se_para_cell{1} = se_S;
else
    se_S = NaN * ones(nr_para,1);
end

% Print results
fprintf('Estimated parameters of GAS tF Model (standard errors in parentheses) \n');

for p = 1:nr_para
    fprintf(' %s: %2.4f (%2.4f) \n',para_cell_str{1}{p},theta_opt(p),se_S(p));
end

fprintf('Maximized LogLikelihood: %8.1f \n', max_LogLik)
disp(' ')








